<?php
/*
ftsdb.classes.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
VERSION: '$Id: ftsdb.classes.php,v 1.42 2011/02/14 17:41:50 schwarz Exp $';
AUTHOR: Lionel Schwarz (lionel.schwarz@in2p3.fr)
*/

class FTSPDO extends PDO {
  private $stmt;
  public function __construct($connString,$connLogin,$connPwd) {
    parent::__construct("oci:dbname=".$connString, $connLogin, $connPwd);
    $this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
  }

  public function prepare($queryString) {
    $res = parent::prepare($queryString);
    $stmt = new FTSPDOStatement($res);
    return $stmt;
  }

  public function getLastDBError() {
    $retArray = $this->errorInfo();
    return $retArray[2];
  }

}

class FTSPDOStatement {
  private $pdo_stmt;
  public $duration;

  public function __construct($pdo_stmt) {
    $this->pdo_stmt = $pdo_stmt;
  }
  public function bind($var, $val) {
    $this->pdo_stmt->bindValue(':' . $var, $val);
  }

  public function execute() {
    $timestart=time();
    //$result = $this->prepared_stmt->execute();
    $res = $this->pdo_stmt->execute();
    $this->duration=time()-$timestart;
    if (!$res)
      throw new Exception(array_pop($pdo_stmt->errorInfo()));
    return new FTS_QueryResult($this->pdo_stmt->fetchAll(PDO::FETCH_ASSOC));
    //$this->prepared_stmt->closeCursor();
  }

  public function __toString() {
    return $this->pdo_stmt->queryString . " (in " . $this->duration . " sec) ";
  }
}

class FTS_QueryResult extends ArrayObject {
  //var $result;
  var $hidden;
  var $maxlines;
  var $query;
  public function __construct($r) {
    //$this->result = $r;
    parent::__construct($r);
    $hidden = Array();
    $this->maxlines=200;
  }

  function getAllRows() {
    return (array) $this;
  } 

  function getNbRows() {
    return $this->count();
  }

  function getRow($n = 1) {
    return $this->offsetGet($n-1);
  }

  function getFirstRow() {
    return $this->getRow(1);
  }

  function hide($colname) {
    $this->hidden[$colname] = True;
  }

  function show($colname) {
    $this->hidden[$colname] = False;
  }

  function setMaxLines($limit) {
    $this->maxlines = $limit;
  }

  function setQuery($query) {
  	 $this->query = $query;
  }
  
  /*function display($id=Null, $expand=False) {
    //return $this->displayArray($this->result, $id, $expand);
    return $this->displayTable($id, $expand);
  }*/

  function display($id=Null, $expand=False) {
    global $ftsm;
    global $errorrepurl;
    global $vo;
    global $channel;
    $maxline = 10;
    //$nrows = count($ar);
    $nrows = $this->count();
    if ($nrows == 0) return 0;
    // maximum number of rows displayed
    // Bug in <PHP 5.3.0 when object is serialized, maxlines is lost
    if ($this->maxlines == 0) $this->maxlines=200;
    if ($nrows > $this->maxlines) $nrows = $this->maxlines;
    echo "\n";
    if ($id != Null)
	echo "<table id=\"$id\" class=\"sortable\" >\n"; 
    else
    	echo "<table>\n";
    print "<thead><TR>";
    foreach ($this->getFirstRow() as $key => $val) {
	if (!$this->hidden[$key]) {
          print "<th><a class=\"sortheader\">" . strtr($key, "_", " ") . "</a></Th>";
	}
    }
    print "</TR></thead>";
      
    print "\n<tbody";
    // $maxline is useless because height is hard-coded in CSS
    if ($maxline > 0) $maxline=10;
    // If $maxline is 0, then no scroll bar
    if ($maxline > 0 && $nrows > $maxline) echo " class=\"scrollable\" ";
    print ">";
    // $alreadyExpanded: there is already a 'expand' image on a previous column
    for ($i = 1; $i <= $nrows; $i++) {
        $alreadyExpanded=false;
	$tr_string = '';
	$tr_class = Null;
	$tr_id = Null;
	// keep T_TRANSFER.REQUEST_ID and T_TRANSFER.TRANSFER_STATE to build log file path
	$currentRow = $this->getRow($i);
        foreach ($currentRow as $c => $v) {
          switch ($c) {
            case "CHANNEL_NAME":
              $td_string = sprintf("<td>");
              if (!$alreadyExpanded and $expand) {
                $td_string .= sprintf("<span><img src=\"plus.gif\" class=\"plus\" id=\"img_%s\"  onClick=\"javascript:channeldetail('$v');\"/></span>", $v, $v);
                $tr_id = $v;
                $alreadyExpanded=true;
              }
              $td_string .= sprintf("<span><a href=\"ftschannel.php?channel=%s\">%s</a></span>", $v, $v);
              $td_string .= sprintf("</td>\n");
             break;
            case "INACTIVE_CHANNELS":
            case "NULL_VO_SHARE":
            case "WAITING_REQUESTS_IN":
              $td_string = sprintf("<td><a href=\"ftschannel.php?channel=%s\">%s</a></td>\n", $v, $v);
              break;
            case "AGENT_NAME":
              $elmts = preg_split("/[:]/", $v);
              if ($elmts[0] != 'channel') {
                $td_string = sprintf("<td>%s</td>\n", $v);
              } else {
                $td_string = sprintf("<td><a href=\"ftschannel.php?channel=%s\">%s</a></td>\n", $elmts[1], $v);
              }
              break;
            case "JOB_ID":
              $td_string = sprintf("<td>");
              if (!$alreadyExpanded) {
		// javascript function is the same as the table ID "activejobdetail" or "endedjobdetail"
                $td_string .= sprintf("<span><img src=\"plus.gif\" class=\"plus\" id=\"img_%s\"  onClick=\"javascript:%s('%s');\"/></span>", $v, $id, $v);
                $tr_id = $v;
                $alreadyExpanded=true;
              }
              $td_string .= sprintf("<span><a href=\"ftsjob.php?jobid=%s\">%s</a></span>", $v, $v);
              $td_string .= sprintf("</td>\n");
              break;
            case "FILE_ID":
              $td_string = sprintf("<td>");
              if (!$alreadyExpanded) {
                $td_string .= sprintf("<span><img src=\"plus.gif\" id=\"img_%s\" class=\"plus\" onClick=\"javascript:filedetail('$v');\"></span>", $v, $v);
                $tr_id = $v;
                $alreadyExpanded=true;
              }
              $td_string .= sprintf("<span>%s</span>", $v);
              $td_string .= sprintf("</td>\n");
              break;
            case "JOB_STATE":
            case "FILE_STATE":
            case "CHANNEL_STATE":
            case "AGENT_STATE":
              $state=explode(' ', $v, 2); // to extract "(x/y)" for "FinishedDirty" jobs
              if ($c == "JOB_STATE") {
                $td_string = sprintf("<td ><a href=\"ftsjobs.php?vo=%s&status=%s%s\">%s</a> %s</td>\n",  
                          $vo, 
                          $state[0], 
                          isset($channel)?"&channel=$channel":"",
                          $state[0], 
                          count($state)==2?$state[1]:'');
              } else {
                $td_string = sprintf("<td > %s</td>\n", $state[0]); 
              }
	           if ($c == "CHANNEL_STATE") {
                $tr_class = ($state[0] == FTS_STATE_ACTIVE || $state[0] == ''?FTS_ALERT_OK:FTS_ALERT_WARNING);
              } else if ($c == "AGENT_STATE") {
                $tr_class = ($state[0] == FTS_STATE_STARTED?FTS_ALERT_OK:FTS_ALERT_ERROR);
              } else {
                $tr_class = $state[0];
              }
              break;
	    case "LOG":
		$request_id = $currentRow['REQUEST_ID'];
		$transfer_state = $currentRow['TRANSFER_STATE'];
		if ($transfer_state == '')
			$transfer_state = $currentRow['LAST_TRANSFER_STATE'];
		if (! is_null($ftsm->log_viewer)) {
                  //$log = $ftsm->log_viewer->getLog('NULL',$request_id,$transfer_state);
  		  //if (! is_null($log)) {
    		    $td_string = sprintf("<td><a href=\"ftslog.php?transfer_id=%s&status=%s\">%s</a></td>",
			$request_id,
			$transfer_state,
			"read log");
  		  //}

		} else {
		// If state is Aborted, => Failed
		  if ($transfer_state == FTS_STATE_PROCESSING) {
			$td_string = sprintf("<td>/var/tmp/glite-url-copy-edguser/%s.log</td>",
					$request_id);
			break;
		  } else {
			if ($transfer_state == FTS_STATE_ABORTED) $transfer_state = FTS_STATE_FAILED;
			list($channel, $dummy) = explode('__', $request_id);
			$td_string = sprintf("<td>/var/tmp/glite-url-copy-edguser/%s%s/%s.log</td>",
						$channel,
						strtolower($transfer_state),
						$request_id);
		  }
		}
              break;
            case "CONTACT":
              $td_string = sprintf("<td><a href=\"mailto:%s\">%s</a></td>\n", $v, preg_replace("/,/",", ",$v));
              break;
            case "DEST_SURL":
              $td_string = sprintf("<td><div class=\"multiline\">%s</div></td>\n", $v);
              break;
            case "SOURCE_SURL":
              $td_string = sprintf("<td><div class=\"multiline\"><a href=\"ftsfile.php?surl=%s\">%s</a></div></td>\n", 
		$v,
		$v);
              break;
            case "SOURCE_FILE":
              $td_string = sprintf("<td><a href=\"ftsfile.php?surl=%s\">%s</a></td>\n", 
		$v,
		strrchr($v, '/'));
              break;
            case "FILE_ERROR_REASON":
            case "TRANSFER_ERROR_REASON":
            case "REASON":
            case "LAST_TRANSFER_ERROR":
              $help='';
              if ($v != '') {
                $v = htmlspecialchars($v); // encode HTML chars
                // remove whitespace, lowercase and replace linefeed by whitespace
                $thisError=preg_replace("/[\n| ]+/"," ",strtolower(trim($v))); 
                foreach ($ftsm->getErrors() as $err) {
                  if (preg_match('/^' . $err->re_desc . '$/', $thisError) == 1) {
                    $help = sprintf("<span class=\"bubble\">%s</span>", $err->explain);
                    break;
                  }
                }
              }
              $td_string = sprintf("<td >%s %s</td>\n", $v, $help);
              break;
            case "FILESIZE": // transform B in KB or MB or GB
              $td_string = sprintf("<td>%s</td>\n", getHumanSize($v));
              break;
            case "THROUGHPUT": // transform B/s in KB/s or MB/s or GB/s
              $td_string = sprintf("<td>%s/s</td>\n", getHumanSize($v));
              break;
            case "CHANNEL_SHARE":
              //printf("<td class=\"%s\"><img src=\"redbox.gif\" height=\"10\" width=\"%s\">  %s %%</td>\n", $errorclass, $v, $v);
              $td_string = sprintf("<td class=\"layout\"><div class=\"" . FTS_ALERT_ERROR . "\" style=\"width: %s\"></div>  %s%%</td>\n", $v, $v);
              break;
            case "MODIFICATION":
              $td_string = "<td>";
              foreach ($v as $field=>$value) {
                $td_string .= sprintf("<div>%s => %s</div>", $field, $value);
              }
              $td_string .= "</td>\n";
              break;
            /*case "TIMEOUT":
              $td_string = sprintf("<td>%s</td>\n", $v);
              break;*/
            case "FROM":
            case "TO":
              // site or group ?
              $group = $ftsm->getGroup($v);
              if (is_null($group)) {
                $td_string = sprintf("<td>%s</td>\n", $v);
              } else {
                $td_string = sprintf("<td>%s</td>\n", $group->toHTML());
              }
              break;
            default:
              $td_string = sprintf("<td>%s</td>\n", $v);
              break;
          }
          if (!$this->hidden[$c]) {
		$tr_string .= $td_string;
	  }
        }

        printf("<tr %s %s>%s</tr>",
		($tr_class != Null)?"class=\"$tr_class\"":"",
		($tr_id != Null)?"id=\"$tr_id\"":"",
		$tr_string);
    }
    print "\n</tbody>";
    echo "</table>\n";
    return $nrows;
  }

  function displayList() {
    global $errorrepurl;
    global $vo;
    global $channel;
    //$ar = $this->result;
    //$nrows = count($ar);
    $nrows = $this->count();
    if ($nrows == 0) return 0;
    echo "<ul>\n";
    for ($i = 1; $i <= $nrows; $i++) {
        foreach ($this->getRow($i) as $c => $v) {
		if ($v != '')		printf("<li><b>%s: </b>%s</li>", $c, $v);
	}
    }
    echo "</ul>\n";
  }
  public function displayXML($tag = 'item', $displayNull = false) {
    //$ar = $this->result;
    //$nrows = count($ar);
    $nrows = $this->count();
    if ($nrows == 0) return 0;
    for ($i = 1; $i <= $nrows; $i++) {
	printf("<%s>", $tag);
        foreach ($this->getRow($i) as $c => $v) {
		if ($v != '' or $displayNull )	printf("<%s>%s</%s>", $c, htmlspecialchars($v), $c);
	}
	printf("</%s>", $tag);
    }
  }

  public function displayPie($keyname, $valuename) {
    try {
      $ftss = new FTSStat($this->query);
      $nrows = $this->count();
      if ($nrows == 0) return 0;
      if ($nrows > $this->maxlines) $nrows = $this->maxlines;
      for ($i = 1; $i <= $nrows; $i++) {
          $thisRow = $this->getRow($i);
  	$ftss->setStat($thisRow[$keyname],(int)$thisRow[$valuename]);
      }

      printf ("%s", $ftss->toPie(800));
    } catch (Exception $e) {
      throw $e;
    }
  }

}


class FTS_JobDetailsResult extends FTS_QueryResult {
  public function __construct($results) {
    $jobs = array();
    $current_job = NULL;
    $current_job_id = -1;
    $total = $total_failed = 0;
    $nrows = count($results);
    $failed_state=FTS_STATE_FINISHED_DIRTY;
    for ($i = 0; $i < $nrows; $i++) {
        $row = $results[$i];
        foreach ($row as $c => $v) {
            switch ($c) {
            case "JOB_ID":
                if ($v != $current_job_id) {
                  if (is_array($current_job)) {
                    if ($current_job["JOB_STATE"] == $failed_state ) 
                        $current_job["JOB_STATE"] = $current_job["JOB_STATE"] . " (" . $total_failed . "/" . $total . ")";
                    $jobs[] = $current_job;
                  }
                  $current_job = array();
                  $current_job[$c] = $v;
                  //printf("changement de job id=%s<br>",$current_job["JOB_ID"]);
                  //}
                  $current_job_id=$v;
                  $total = $total_failed = 0;
                }
                break;
            case "FILE_STATE":
                if ($v == FTS_STATE_FAILED) $total_failed++;
                $total++;
                break;
            default:
                $current_job[$c] = $v;
                break;
            }
        }
    }
    if ($current_job != NULL) {
      $jobs[] = $current_job;
    }
    parent::__construct($jobs);
  }

  public function display($id = "jobdetail") {
    return parent::display($id);
  }
}

class FTSPDOQuery {
  public $query;
  public $duration;
  public function __construct($queryString='') {
    $this->query = $queryString;
  }

  public function setQuery($queryString) {
    $this->query = $queryString;
  }

  public function execute($expire=null) {
    global $ftsm;
    if (is_null($expire)) $expire=$ftsm->conf->refresh;
    try {
      $resultobj = ftsCacheReadQuery($this->query,$expire);
    } catch (Exception $e) {
      global $ftsconn;
      $timestart=time();
      $result = $ftsconn->query($this->query);
      $this->duration=time()-$timestart;
      if ($result === False)
        throw new Exception($ftsconn->getLastDBError());
      ftsSyslog(sprintf("EXECUTED %s IN %d s\n", $this->query, $this->duration), LOG_DEBUG);
      $resultobj = new FTS_QueryResult($result->fetchAll(PDO::FETCH_ASSOC));
      ftsCacheWriteQuery($this->query,$resultobj);
    }
    return $resultobj;
  }

}

class FTS_ChannelListResult extends FTS_QueryResult {
  public function display($channel) {
    echo ' channel: <select name="channel" onChange="javascript:this.form.submit();">';
  foreach ($this->getAllRows() as $row) {
     printf ("<option value=\"%s\" %s>%s", $row['CHANNEL_NAME'], ($row['CHANNEL_NAME'] == $channel)?"selected":"", $row['CHANNEL_NAME']);
  }
  echo '</select>';
  }
}

class ChannelList {
  public $result;
  private $withNone;
  private $withAll;
  public function __construct() {
    try {
    	$this->result = ftsCacheRead('channellist',3600);
    } catch (Exception $e) {
      global $ftsconn;
      global $ftsm;
      $query = "select CHANNEL_NAME from " . $ftsm->conf->connSchema . "T_CHANNEL order by CHANNEL_NAME";
      $stmt = $ftsconn->query($query);
      $this->result = $stmt->fetchAll();
      ftsCacheWrite('channellist',$this->result);
    }
    $this->withAll = False;
    $this->withNone = False;
  }
  public function addAll() {
    $this->withAll = True;
  }
  public function addNone() {
    $this->withNone = True;
  }
  public function toHTML($channel) {
    $html = ' channel: <select name="channel" onChange="javascript:this.form.submit();">';
    if ($this->withAll === True) {
      $html .= '<option value="' . FTS_ALL . '"';
      if ($channel === FTS_ALL) $html .= " selected";
      $html .= '>All</option>';
    }
    foreach ($this->result as $row) {
      $opt = sprintf ("<option value=\"%s\" %s>%s</option>", 
	$row['CHANNEL_NAME'], 
	($row['CHANNEL_NAME'] == $channel)?"selected":"", 
	$row['CHANNEL_NAME']);
      $html .= $opt;
    }
    if ($this->withNone === True) {
      $html .= '<option value="' . FTS_NONE . '"';
      if ($channel === FTS_NONE) $html .= " selected";
      $html .= '>None</option>';
    }
    $html .= '</select>';
    return $html;
  }
}

?>
