<?php
/*
ftshttplogviewer.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
* ftshttplogviewer.php
*
* Plugin for reading log files on a web server.
* 
* @version '$Id: ftshttplogviewer.php,v 1.5 2010/09/17 09:05:29 schwarz Exp $'
* @author Lionel Schwarz <lionel.schwarz@in2p3.fr>
*/

require_once('ftslog.classes.php');

/**
 * This class allows to read a log file located on a web server.
 *
 * @package TransferLog
 * @see FTSLogViewer
 * @see FTSLog
 */
class FTSHTTPLogViewer extends FTSLogViewer {

  /**
  * Constructor
  *
  * @param string $host Host : where the log was written
  */
  public function __construct($host) {
    if (!function_exists('curl_init')) 
      throw new LogViewerInstantiationException("php is not compiled with cURL support");
    parent::__construct($host);
  }

  /**
  * This method reads the log in a file located on a web server
  *
  * The root directory <ROOT> is defined in the configuration file.
  * The URL will be "http://<ROOT>/<DATE>/<CHANNEL><STATE_IN_LOWERCASE>/<IDENTIFIER>.log" with <DATE> is the transfer date formatted as "YYYY-MM-DD"
  *
  * @return Array : an array with each row containg a line of the log file
  * @see FTSLog
  */
  public function retrieveLog() {
    $logurl = "http://" . $this->root;
    // Add Date directory YYYY-MM-DD
    $logurl .= '/' . strftime('%Y-%m-%d', $this->log->getTransferDate());
    // Add Channel dir and log name CHANNELstate/transfer_id.log
    $logurl .= '/' . $this->log->getArchivedFileName();
    $curl_session = curl_init();
    curl_setopt($curl_session, CURLOPT_URL, $logurl);
    curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, 1);
    //curl_setopt($curl_session, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl_session, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl_session, CURLOPT_CONNECTTIMEOUT, 5);
    curl_setopt($curl_session, CURLOPT_FAILONERROR, True);
    $result = curl_exec($curl_session);
    $curl_errno = curl_errno($curl_session);
    $curl_error = curl_error($curl_session);
    curl_close($curl_session);
    if ($result === false) {
      $err= sprintf("Could not download URL <b>%s</b> : %s", $logurl, $curl_error);
      ftsSyslog($err,	LOG_WARNING);
      //return null;
      if ($curl_errno == 22) {
        throw new Exception ($err . " The file does not exist on the server");
      } else {
        throw new Exception ($err);
      }
    }
    return explode("\n", $result);

  }

}

