<?php
/*
ftsjobs.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
VERSION: '$Id: ftsjobs.php,v 1.14 2010/08/26 14:22:45 schwarz Exp $';
AUTHOR: Lionel Schwarz (lionel.schwarz@in2p3.fr)
*/

require_once('ftsheader.php');

if (strpos($_SERVER['HTTP_USER_AGENT'], 'Safari')) {
  echo '<script language="JavaScript" type="text/javascript" src="detail-safari.js">';
} else {
  echo '<script language="JavaScript" type="text/javascript" src="detail.js">';
}
echo '</script>';

try {
  echo '<div class="filter">';
  $channel = $_GET['channel'];
  if (!isset($channel)) $channel = FTS_ALL;
  if ($channel != FTS_ALL && !isValidChannel($channel)) {
    throw new InvalidInputException( "Not a valid channel name: $channel");
  }
  $status = $_GET['status'];
  if (!isset($status)) $status = FTS_STATE_ACTIVE;
  else if (!$ftsm->isValidState($status)) {
    throw new InvalidInputException("Not a valid FTS job state: $status");
  }
  $vo = $_GET['vo'];
  if (!isset($vo)) $vo = FTS_ALL;
  else if (!$ftsm->conf->isKnownVo($vo)) $vo= FTS_ALL;
  if ($vo == "") $vo = FTS_ALL;


  echo '<form action="' . $_SERVER['PHP_SELF'] . '" method="get">';
  echo 'Filter by ';
  $cl = new ChannelList();
  $cl->addAll();
  $cl->addNone();
  print ($cl->toHTML($channel));

  echo ' for jobs in state <select name="status" onChange="javascript:this.form.submit();">';
  foreach ($ftsm->getStates() as $state) {
    printf ("<option value=\"%s\" %s>%s", $state, $state==$status?"selected":"", $state);
  }
  echo '           </select>';


  echo ' for VO ';
  print ($ftsm->conf->voToHTML($vo));
  

  $view = "jobs";

  echo '  <input type="submit">';
  echo '</form>';
  echo '</div>';
  
  if ($status != FTS_STATE_ACTIVE && $status != FTS_STATE_READY) {
    echo "<h3>$status jobs (last 12 hours)</h3>";
  } else {
    echo "<h3>current $status jobs</h3>";
  }

  if ($view == "jobs") {
    $query = "select T_JOB.JOB_ID, JOB_STATE, FILE_STATE, " . getTimeInSql('SUBMIT_TIME') . " ";
    if ($status != FTS_STATE_ACTIVE and $status != FTS_STATE_READY) {
      $query .= ", " . getTimeInSql('T_JOB.JOB_FINISHED');
    }
    $query .= ", USER_DN";
    //$query = "select JOB_ID, USER_DN";
    if ($channel == FTS_ALL) 
      $query .= " ,CHANNEL_NAME ";
    if ($vo == FTS_ALL) 
      $query .= " ,VO_NAME ";
    //$query .= " ,JOB_STATE ";
    $query .= " from " . $ftsm->conf->connSchema . "T_JOB, " . $ftsm->conf->connSchema . "T_FILE ";
    if ($status != "Slow") { // show Active jobs since a long time
        $query .= " where JOB_STATE = '" . $status . "' ";
    } else {
        $query .= " where JOB_STATE = '" . FTS_STATE_ACTIVE . "' ";
    }
    $query .= " and T_JOB.JOB_ID = T_FILE.JOB_ID ";
    if ($vo != FTS_ALL) 
      $query .= " and VO_NAME = '" . $vo . "' ";
    if ($channel == FTS_NONE) {
      $query .= " and CHANNEL_NAME is NULL ";
    } else if ($channel != FTS_ALL) {
      $query .= " and CHANNEL_NAME = '" . $channel . "' ";
    }
    if ($status != FTS_STATE_ACTIVE && $status != FTS_STATE_READY) {
      $query .= " and (T_JOB.SUBMIT_TIME > current_timestamp - INTERVAL '12' hour) ";
      $query .= " order by T_JOB.SUBMIT_TIME desc ";
    } else {
      $query .= " order by T_JOB.SUBMIT_TIME asc ";
    }
    $ftsq->setQuery($query);
    //print $query;
    $r = $ftsq->execute();

    $jdr = new FTS_JobDetailsResult($r->getAllRows());
    $jdr->hide('JOB_STATE');
    $jdr->display();
  } else {
    $query .= "select T_FILE.FILE_ID, T_JOB.JOB_ID, T_FILE.FILE_STATE, T_FILE.JOB_FINISHED, T_FILE.SOURCE_SURL as SOURCE_FILE ";
    $query .= ", T_FILE.REASON ";
    $query .= " from " . $ftsm->conf->connSchema . "T_FILE, " . $ftsm->conf->connSchema . "T_JOB ";
    $query .= " where T_FILE.FILE_STATE = '" . $status . "' ";
    if ($status != FTS_STATE_ACTIVE && $status != "Slow") {
      $query .= " and (T_FILE.FINISH_TIME > current_timestamp - INTERVAL '12' hour) ";
    }
    $query .= " and T_FILE.JOB_ID = T_JOB.JOB_ID ";
    if ($vo != FTS_ALL) 
      $query .= " and T_JOB.VO_NAME = '" . $vo . "' ";
    if ($channel != FTS_ALL)
      $query .= " and T_JOB.CHANNEL_NAME = '" . $channel . "' ";
    $query .= " order by T_FILE.FINISH_TIME desc";
    $ftsq->setQuery($query);
    $r = $ftsq->execute();
    $r->display('jobdetail');
  }

} catch (Exception $e) {
  die('<span class="' . FTS_CSS_CLASS_ALERT . '">ERROR: --' . $e->getMessage() . '--</span>');
}
echo '</div>';
footer();
?>