<?php
/*
ftsmonitor.classes.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
VERSION: '$Id: ftsmonitor.classes.php,v 1.56 2011/04/18 08:51:28 schwarz Exp $';
AUTHOR: Lionel Schwarz (lionel.schwarz@in2p3.fr)
*/

$ftsdebug=False;

class FTSMonitorConf {
  // Defined in ftsmonitor.conf
  public $siteName;
  public $vos;
  public $connString;
  public $connLogin;
  public $connPwd;
  public $connSchema;
  public $cache_dir;
  public $refresh;
  public $timescales;
  public $timescale;
  public $css;
  public $bdii_host;
  public $bdii_port;
  public $bdii_fts_host;
  public $errors_csv_file;
  public $jpgraph_path;
  public $log_rootdir;
  public $time_utc;
  public $time_format;

  // Calculated
  public $currentSchema;

  public function __construct() {
    $this->currentSchema = null;
    $this->siteName = "UNDEFINED";
    $this->vos = Array();
    $this->refresh = 600;
    $this->timescales = Array("1", "2", "6", "12");
    $this->timescale = "12";
    $this->css = "ftsmonitor.css";
    $this->bdii_host = NULL;
    $this->bdii_port = "2170";
    $this->bdii_fts_host = NULL;
    $this->errors_csv_file = "ftserrors.csv";
    if ($_SERVER["FTSMONITOR_CONFIG"]) 
        $configfile = $_SERVER["FTSMONITOR_CONFIG"] ;
    else 
        $configfile = "private/ftsmonitor.conf" ;
    $list = parse_ini_file($configfile);
    $this->jpgraph_path = 'jpgraph';
    $this->log_rootdir = NULL;
    $this->log_viewername = NULL;
    $this->time_utc = false;
    $this->time_format = 'DD/MM/YYYY HH24:MI:SS TZR';
    $this->cache_dir = 'cache';
    //print_r($ini_array);
    if (empty($list['vo']) === False) $this->vos = $list['vo'];
    $this->siteName = $list['sitename'];
    $this->connString = $list['connString'];
    $this->connLogin = $list['connLogin'];
    $this->connPwd = $list['connPwd'];
    $this->connSchema = $list['connSchema'];
    if (!empty($list['refresh'])) $this->refresh = $list['refresh'];
    if (!empty($list['timescales'])) $this->timescales = $list['timescales'];
    if (!empty($list['timescale'])) $this->timescale = $list['timescale'];
    if (!empty($list['css'])) $this->css = $list['css'];
    if (!empty($list['bdii_host'])) $this->bdii_host = $list['bdii_host'];
    if (!empty($list['bdii_port'])) $this->bdii_port = $list['bdii_port'];
    if (!empty($list['bdii_fts_host'])) $this->bdii_fts_host = $list['bdii_fts_host'];
    if (!empty($list['jpgraph_path'])) $this->jpgraph_path = $list['jpgraph_path'];
    if (!empty($list['log_rootdir'])) $this->log_rootdir = $list['log_rootdir'];
    if (!empty($list['log_viewername'])) $this->log_viewername = $list['log_viewername'];
    if (!empty($list['time_utc'])) $this->time_utc = $list['time_utc'];
    if (!empty($list['time_format'])) $this->time_format = $list['time_format'];
    if (!empty($list['cache_dir'])) $this->cache_dir = $list['cache_dir'];
  }

  public function voToHTML($vo=FTS_ALL) {
    $html = '<select name="vo" onChange="javascript:this.form.submit();">';
    /*foreach ($this->vos as $vocode => $voobj) {
      $html .= sprintf ("<option value=\"%s\" %s>%s", $vocode, ($vocode == $vo)?"selected":"", $voobj->name);
    }*/
    foreach ($this->vos as $vocode) {
      $html .= sprintf ("<option value=\"%s\" %s>%s", $vocode, ($vocode == $vo)?"selected":"", $vocode);
    }
    $html .= sprintf("<option value=\"%s\" %s>%s", FTS_ALL, (FTS_ALL == $vo)?"selected":"", 'All');
    $html .= ' </select>';
    return $html;
  }

  public function isKnownVo($vo) {
    return in_array($vo, $this->vos);
  }

  public function validateTimescale($t) {
    if (! is_numeric($t)) {
      return $this->timescale;
    } else if (! in_array($t, $this->timescales)) {
      return $this->timescale;
    } else {
      return $t;
    }
  }

  public function timescalesToHTML($timescale) {
    $html =  '<select name="timescale" onChange="javascript:this.form.submit();">';
    foreach ($this->timescales as $delay) {
      $html .= sprintf ("<option value=\"%s\" %s>%s h", $delay, $delay==$timescale?"selected":"", $delay);
    }
    $html .=  '           </select>';
    return $html;
  }
}

class FTSMonitor {
  CONST VERSION = "1.5.3";
  private $ftsdbschema_version_supported;
  private $states;
  public $conf;
  //public $conn;
  private $errors;
  private $sitegroups;
  public $log_viewer;

  public function __construct() {
    $this->ftsdbschema_version_supported=Array("3.1.0", "3.2.0", "3.3.0", "3.4.0", "3.4.1");
    $this->conf = new FTSMonitorConf();
/*    $this->conn = new FTSPDO($this->conf->connString,
                          $this->conf->connLogin,
                          $this->conf->connPwd);*/
    $this->states = Array(FTS_STATE_READY,
			FTS_STATE_ACTIVE,
			FTS_STATE_FINISHED,
			FTS_STATE_FINISHED_DIRTY,
			FTS_STATE_FAILED,
			FTS_STATE_CANCELED);
    $this->errors = getErrorsCsv($this->conf->errors_csv_file);
    $this->sitegroups = Array();
    $this->log_viewer = NULL;
    if (! is_null($this->conf->log_viewername)) {
      $logviewerfile = strtolower($this->conf->log_viewername) . '.php';
      try {
        if (!file_exists($logviewerfile)) 
          throw new LogViewerInstantiationException("File not found: $logviewerfile");
        // Done by autoload in ftsinit.php
        //require_once ($logviewerfile);
        if (!class_exists($this->conf->log_viewername))
          throw new LogViewerInstantiationException("Class not found: $this->conf->log_viewername");
        $reflect = new ReflectionClass($this->conf->log_viewername);
        if (! is_null($this->conf->log_rootdir)) {
          $this->log_viewer = $reflect->newInstance($this->conf->log_rootdir);
        } else {
          $this->log_viewer = $reflect->newInstance();
        }
      } catch (LogViewerInstantiationException $lvie) {
      }
    }
  }

  public function getSupportedSchemas() {
    return $this->ftsdbschema_version_supported;
  }

  public function getStates() {
    return $this->states;
  }

  public function getErrors() {
    return $this->errors;
  }

  public function isValidState($st) {
    return in_array($st, $this->states);
  }

  public function __toString() {
    return "FTS Monitor v" . self::VERSION;
  }

/*  public function getLastDBError() {
    $retArray = $this->conn->errorInfo();
    return $retArray[2];
  }*/

  public static function getPackageName() {
    return "ftsmonitor-" . self::VERSION;
  }

  public function addSiteGroup($g, $s) {
    if (array_key_exists($g, $this->sitegroups)) {
      //$this->sitegroups[$g]->addSite($s);
      $this->sitegroups[$g]->append($s);
    } else {
      $group = new FTSSiteGroup($g);
      //$group->addSite($s);
      $group->append($s);
      $this->sitegroups[$g] = $group;
    }
  }

  public function getGroup($g) {
    //$g = '[' . $g . ']';
    if (array_key_exists($g, $this->sitegroups)) {
      return $this->sitegroups[$g];
    } else {
      return null;
    }
  }
}

class FTSChannel {
  var $name;
  var $nbtrfailed;
  var $nbtrok;
  public function __construct($c) {
    $this->name = $c;
    $this->nbtrfailed = $this->nbtrok = 0;
  }
  public function addTransferResult($res) {
    if ($res) $this->nbtrok++;
    else      $this->nbtrfailed++;
  }
  public function getEfficiency() {
    if ($this->nbtrok+$this->nbtrfailed == 0) return -1;
    return ($this->nbtrok/($this->nbtrok+$this->nbtrfailed));
  }
}

class FTSSiteGroup extends ArrayObject {
  var $name;
  public function __construct($name) {
    parent::__construct();
    $this->name = $name;
  }

  public function toHTML() {
    $html = sprintf("<div class=\"sitegroup\">group %s members:</div>", $this->name) ;
    $html .= "<div class = \"sites\">";
    $iterator = $this->getIterator();

    while($iterator->valid()) {
      $html .=  $iterator->current() . ", ";
      $iterator->next();
    }
    $html .= "</div>";
    return $html;
  }
}

/*
class FTSSiteGroup {
  var $sites;
  var $name;
  public function __construct($name) {
    $this->name = $name;
    $this->sites = Array();
  }
  public function addSite($s) {
    if (! in_array($s, $this->sites)) {
      $this->sites[] = $s;
    }
  }
  public function toHTML() {
    $html = sprintf("<div class=\"sitegroup\">group %s members:</div>", $this->name) ;
    $html .= "<div class = \"sites\">";
    foreach ($this->sites as $s) {
      $html .=  $s . ", ";
    }
    $html .= "</div>";
    return $html;
  }
}
*/

class FTSAlarm {
   var $component;
   var $msg;
   var $level;
   var $link;
   var $tip;
   function __construct($msg, $level = E_ERROR, $tip = null, $link = null) {
     //$this->component = $comp;
     $this->msg = $msg;
     $this->level = $level;
     $this->link = $link;
     $this->tip = $tip;
   }
   function toHTML() {
     /*if ($this->link) {
       $tr .= '<td><a href="' . $this->link . '">' . $this->component . "</a></td>";
     } else {
       $tr .= "<td>" . $this->component . "</td>";
     }*/
     switch ($this->level) {
       case E_ERROR:
         $level=FTS_ALERT_ERROR;
         break;
       case E_WARNING:
         $level=FTS_ALERT_WARNING;
         break;
       case E_NOTICE:
         $level=FTS_ALERT_OK;
         break;
     }
     return sprintf("\n<tr class=\"%s\"><td>%s%s%s %s</td></tr>",
                    $level,
                    is_null($this->link)?'':'<a href="' . $this->link . '">',
                    $this->msg,
                    is_null($this->link)?'':'</a>',
                    is_null($this->tip)?'':'<span class="bubble">' . $this->tip . '</span>');
   }
}

abstract class FTSSensor {
  var $name;
  var $link;
  var $vo;
  var $alarms = Array();
  //var $lastRun;
  function __construct ($vo = NULL) {
    $this->name = $this->getName();
    $this->link = $this->getLink();
    if ($vo == FTS_ALL) $this->vo = NULL;
    else              $this->vo = $vo;
  }
  function getName() {
    return get_class($this);
  }
  function getLink() {
    return NULL;
  }
  public function run() {
  }
  public function execute() {
    //global $conf;
    //if (!in_array($conf->state, $this->getStates())) return;
    return $this->run();
  }
  function getAlarms() {
    //print $this->name . count($this->alarms) . '<br>';
    return $this->alarms;
  }
}

class FTSChannelSensor extends FTSSensor {
  public function run() {
    global $ftsm;
    global $ftsq;

    $query = " select CHANNEL_NAME";
    $query .= " from " . $ftsm->conf->connSchema . "T_CHANNEL ";
    $query .= " where 'channel:'||CHANNEL_NAME NOT IN ( ";
    $query .= " SELECT agent_name from " . $ftsm->conf->connSchema . "T_AGENT where AGENT_TYPE = 'channel')";
    $ftsq->setQuery($query);
    $r = $ftsq->execute(7200);
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows > 0) {
      foreach ($results as $line) {
      $this->alarms[] = new FTSAlarm(sprintf("No agent found for channel %s", 
                                             $line['CHANNEL_NAME']),
                                     E_ERROR);
      }
    }

    $query = "select CHANNEL_NAME, CHANNEL_STATE, MESSAGE from " . $ftsm->conf->connSchema . "T_CHANNEL where CHANNEL_STATE != '" . FTS_STATE_ACTIVE . "'";
    $ftsq->setQuery($query);
    $r = $ftsq->execute();
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows == 0) {
      $this->alarms[] = new FTSAlarm("All channels are '" . FTS_STATE_ACTIVE . "'", E_NOTICE);
    } else {
      foreach ($results as $line) {
        $msg = sprintf("<a href=\"ftschannel.php?channel=%s\">%s</a>", 
		is_null($this->vo)?$line['CHANNEL_NAME']:$line['CHANNEL_NAME'] . "&vo=" . $this->vo,
		$line['CHANNEL_NAME']);
        $this->alarms[] = new FTSAlarm(sprintf("Channel %s is %s (%s)", 
                                               $msg,
                                               $line['CHANNEL_STATE'],
                                               $line['MESSAGE']),
                                       $line['CHANNEL_STATE'] == FTS_STATE_HALTED?E_ERROR:E_WARNING);
      }
    }

    $query = " select T_CHANNEL.CHANNEL_NAME, CHANNEL_SHARE, CHANNEL_VOSTATE ";
    if (is_null($this->vo)) {
        $query .= " ,T_CHANNEL_VO_SHARE.VO_NAME";
    }
    $query .= " from " . $ftsm->conf->connSchema . "T_CHANNEL, " . $ftsm->conf->connSchema . "T_CHANNEL_VO_SHARE ";
    $query .= " where T_CHANNEL.CHANNEL_NAME = T_CHANNEL_VO_SHARE.CHANNEL_NAME ";
    $query .= " and (";
    $query .= "      T_CHANNEL_VO_SHARE.CHANNEL_SHARE <= 0 ";
    $query .= "      OR T_CHANNEL_VO_SHARE.CHANNEL_VOSTATE != '" . FTS_STATE_ON . "' ";
    $query .= "     )";
    if (! is_null($this->vo)) {
        $query .= " and T_CHANNEL_VO_SHARE.VO_NAME = '" . $this->vo . "'";
    }
    $ftsq->setQuery($query);
    $r = $ftsq->execute(3600);
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows > 0) {
      foreach ($results as $line) {
        $msg = sprintf("<a href=\"ftschannel.php?channel=%s\">%s</a>", 
		is_null($this->vo)?$line['CHANNEL_NAME']:$line['CHANNEL_NAME'] . "&vo=" . $this->vo,
		$line['CHANNEL_NAME']);
        if ($line['CHANNEL_VOSTATE'] != FTS_STATE_ON) {
          $this->alarms[] = new FTSAlarm(sprintf("%s state on %s is %s", 
                                  $line['VO_NAME'], 
                                    $msg,
                                    $line['CHANNEL_VOSTATE']),
                            E_WARNING);
        } else {
          $this->alarms[] = new FTSAlarm(sprintf("%s share on %s is %s", 
                                  $line['VO_NAME'], 
                                    $msg,
                                    $line['CHANNEL_SHARE']),
                            E_WARNING);
        }
      }
    }

  }
}

class FTSJobSensor extends FTSSensor {
  public function run() {
    global $ftsm;
      $query = "select CHANNEL_NAME from " . $ftsm->conf->connSchema . "T_CHANNEL ";
      $query .= "where CHANNEL_STATE = '" . FTS_STATE_ACTIVE . "' ";
      $query .= "and CHANNEL_NAME in ( ";
      $query .= "  select T_CHANNEL_VO_SHARE.CHANNEL_NAME from " . $ftsm->conf->connSchema . "T_JOB, " . $ftsm->conf->connSchema . "T_CHANNEL_VO_SHARE ";
      $query .= "  where JOB_STATE='" . FTS_STATE_READY . "' and CANCEL_JOB is null and CHANNEL_SHARE > 0 ";
      $query .= "  and T_JOB.VO_NAME = T_CHANNEL_VO_SHARE.VO_NAME ";
      if (! is_null($this->vo)) {
        $query .= " and T_CHANNEL_VO_SHARE.VO_NAME = '" . $this->vo . "'";
      }
      $query .= "  and T_JOB.CHANNEL_NAME = T_CHANNEL_VO_SHARE.CHANNEL_NAME ";
      $query .= "  group by T_CHANNEL_VO_SHARE.CHANNEL_NAME) ";
      $query .= "and CHANNEL_NAME not in ( ";
      $query .= "  select T_CHANNEL_VO_SHARE.CHANNEL_NAME from " . $ftsm->conf->connSchema . "T_JOB, " . $ftsm->conf->connSchema . "T_CHANNEL_VO_SHARE ";
      $query .= "  where JOB_STATE='" . FTS_STATE_ACTIVE . "' and CHANNEL_SHARE > 0 ";
      $query .= "  and T_JOB.VO_NAME = T_CHANNEL_VO_SHARE.VO_NAME ";
      if (! is_null($this->vo)) {
        $query .= " and T_CHANNEL_VO_SHARE.VO_NAME = '" . $this->vo . "'";
      }
      $query .= "  and T_JOB.CHANNEL_NAME = T_CHANNEL_VO_SHARE.CHANNEL_NAME ";
      $query .= "  group by T_CHANNEL_VO_SHARE.CHANNEL_NAME) ";
//    $r = null;
//    try {
//      $r = ftsCacheReadQuery($query,$ftsm->conf->refresh);
//    } catch (Exception $e) {
      global $ftsq;

      $ftsq->setQuery($query);
      $r = $ftsq->execute();
//      ftsCacheWriteQuery($query, $r);
//    }
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows > 0) {
      foreach ($results as $line) {
        $alarmTip = "There is no '" . FTS_STATE_ACTIVE . "' jobs but some jobs are '" . FTS_STATE_READY . "'";
        $this->alarms[] = new FTSAlarm(sprintf("%s has " . FTS_STATE_READY . " requests", $line['CHANNEL_NAME']),
                                       E_WARNING,
                                       $alarmTip);
      }
    }
  }
}

class FTSAgentSensor extends FTSSensor {
  public function run() {
    global $ftsm;
    $query = "select AGENT_NAME, AGENT_STATE, AGENT_TYPE from " . $ftsm->conf->connSchema . "T_AGENT, " . $ftsm->conf->connSchema . "T_CHANNEL ";
    $query .= " where AGENT_NAME = 'channel:' || CHANNEL_NAME ";
    $query .= " and CHANNEL_STATE != '" . FTS_STATE_ARCHIVED . "'";
    $query .= " and AGENT_STATE != '" . FTS_STATE_STARTED . "'";
    global $ftsq;

    $ftsq->setQuery($query);
    $r = $ftsq->execute(300);
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows == 0) {
      $this->alarms[] = new FTSAlarm("All agents are '" . FTS_STATE_STARTED . "'", E_NOTICE);
    } else {
      foreach ($results as $line) {
        $agent_name = explode(':',$line['AGENT_NAME']);
        $agent_name = $agent_name[1];
        $msg = $agent_name;
        if ($line['AGENT_TYPE'] == 'channel') {
          $msg = sprintf("<a href=\"ftschannel.php?channel=%s\">%s</a>", 
		$agent_name,
		$agent_name);
        }
        $this->alarms[] = new FTSAlarm(sprintf("Agent %s is %s", 
                                               $msg,
                                               $line['AGENT_STATE']),
                                       E_ERROR);
      }
    }
  }
}

class FTSVoSensor extends FTSSensor {
  public function run() {
    global $ftsm;

    $voagents = null;
    try {
      $voagents = ftsCacheRead('voagents',3600);
    } catch (Exception $e) {
      global $ftsq;
      $query = "select AGENT_NAME from " . $ftsm->conf->connSchema . "T_AGENT where AGENT_TYPE = 'vo'";
      $ftsq->setQuery($query);
      $r = $ftsq->execute();
      $voagents = Array();
      foreach ($r->getAllRows() as $row) {
        $voagents[] = $row['AGENT_NAME'];
      }
      ftsCacheWrite('voagents',$voagents);
    }
    foreach ($ftsm->conf->vos as $v) {
      if (!isset($this->vo) or $this->vo == $v) {
        if (!in_array('vo:' . $v, $voagents)) {
          $this->alarms[] = new FTSAlarm("No VO agent found for %s" . $v , E_ERROR);
        }
      }
    }
  }
}

class FTSEfficiencySensor extends FTSSensor {
  public function run() {
    global $ftsm;
//    $r = null;
      $query = "select FILE_ID, FILE_STATE, CHANNEL_NAME from " . $ftsm->conf->connSchema . "T_FILE, " . $ftsm->conf->connSchema . "T_JOB where T_FILE.FINISH_TIME > current_timestamp - INTERVAL '1' hour and t_file.job_id = t_job.job_id";
      if (isset($this->vo)) {
        $query .= " and vo_name = '" . $this->vo . "'";
      }
      $query .= " ORDER BY CHANNEL_NAME ";
//    try {
//      $r = ftsCacheReadQuery($query,$ftsm->conf->refresh);
//    } catch (Exception $e) {
      global $ftsq;
      $ftsq->setQuery($query);
      $r = $ftsq->execute();
//      ftsCacheWriteQuery($query,$r);
//    }

    $channels = Array();
    foreach ($r->getAllRows() as $row) {
      if ($row['CHANNEL_NAME'] != '') {
        $currentChannel = $row['CHANNEL_NAME'];
        $currentState = $row['FILE_STATE'];
        //$transferRes = ($currentState == 'Finished' or $currentState == 'Done');
        if (! array_key_exists($currentChannel, $channels)) {
          $channels[$currentChannel] = new FTSChannel($currentChannel);
        }
        if ($currentState != FTS_STATE_ACTIVE and $currentState != FTS_STATE_CANCELED and $currentState != FTS_STATE_WAITING) { 
          $channels[$currentChannel]->addTransferResult($currentState == FTS_STATE_FINISHED or $currentState == FTS_STATE_DONE);
        }
      }
    }
    foreach ($channels as $c) {
      if ($c->getEfficiency() >= 0) {
        $effPourc = $c->getEfficiency()*100;
        if ($effPourc < 75) {
          $alarmMsg = sprintf("<a href=\"ftschannel.php?channel=%s\">%s</a> efficiency: %d %%", 
				is_null($this->vo)?$c->name:$c->name . "&vo=" . $this->vo,
				$c->name, 
				$effPourc);
          $alarmTip = sprintf("%d %% of the transfers finished in the last hour failed", 100-$effPourc);
          $this->alarms[] = new FTSAlarm($alarmMsg, E_ERROR, $alarmTip);
        }
      }
    }
  }
}

class FTSBDIISensor extends FTSSensor {
  public function run() {
    global $ftsm;

    if ($ftsm->conf->bdii_host === null or $ftsm->conf->bdii_fts_host === null) return;
    if (!function_exists('ldap_connect')) {
            $this->alarms[] = new FTSAlarm("Could not query BDII", E_WARNING, 'LDAP module is missing');
            return;
    }

//    $r = null;
      $query = " select CHANNEL_NAME";
      $query .= " from " . $ftsm->conf->connSchema . "T_CHANNEL ";
      $query .= " where CHANNEL_STATE = '" . FTS_STATE_ACTIVE . "'";
//    try {
//      $r = ftsCacheReadQuery($query,$ftsm->conf->refresh);
//    } catch (Exception $e) {
      global $ftsq;

      $ftsq->setQuery($query);
      $r = $ftsq->execute(3600);
//      ftsCacheWriteQuery($query,$r);
//    }
    $results = $r->getAllRows();
    $nrows = $r->getNbRows();

    if ($nrows > 0) {
      $ldap_channels = null;
      try {
        $ldap_channels = ftsCacheRead('channels.bdii',3600);
      } catch (Exception $e) {
        $ds=ldap_connect($ftsm->conf->bdii_host,$ftsm->conf->bdii_port);
        if (!$ds) {
          $this->alarms[] = new FTSAlarm('Cannot connect to ' . $ftsm->conf->bdii_host . ':' . $ftsm->conf->bdii_port, E_WARNING);
          return;
        }

        $r=ldap_bind($ds);
        if (!$r) {
          $this->alarms[] = new FTSAlarm('Cannot connect to ' . $ftsm->conf->bdii_host . ':' . $ftsm->conf->bdii_port, E_WARNING);
          return;
        }
        $dn="Mds-Vo-name=" . $ftsm->conf->siteName . ",o=grid";
        $filter="(GlueServiceUniqueID=*_" . $ftsm->conf->bdii_fts_host . "_org.glite.ChannelAgent)";
        $whattoget=array("GlueServiceUniqueID");
        $sr=@ldap_search($ds,$dn,$filter,$whattoget); 
        if (!$sr) {
            $this->alarms[] = new FTSAlarm("Request error on BDII", E_WARNING, $dn);
            return;
        }
        $ldap_channels=Array();
        for ($entryID=ldap_first_entry($ds,$sr);
              $entryID!=false;
              $entryID=ldap_next_entry($ds,$entryID))
        {
            $values = ldap_get_values($ds,$entryID,"GlueServiceUniqueID");
            $ldap_channels[] = $values[0];
        }
        ftsCacheWrite('channels.bdii',$ldap_channels);
      }
      //var_dump($ldap_channels);
      foreach ($results as $channel) {
        if (!in_array($channel['CHANNEL_NAME'] . "_" . $ftsm->conf->bdii_fts_host . "_org.glite.ChannelAgent", $ldap_channels)) {
            $this->alarms[] = new FTSAlarm($channel['CHANNEL_NAME'] . " is not published", E_WARNING);
        }
      }
    }
  }
}

class FTSHistoryLogSensor extends FTSSensor {
  public function run() {
    global $ftsm;
    if ($ftsm->conf->currentSchema >= "3.4.0") {
      $query = " select * from " . $ftsm->conf->connSchema . "t_history_log where finishtime = (select max(finishtime) from " . $ftsm->conf->connSchema . "t_history_log)";
      global $ftsq;

      $ftsq->setQuery($query);
      $r = $ftsq->execute(-1);
      $nrows = $r->getNbRows();
      if ($nrows > 0) {
        $results = $r->getFirstRow();
        if ($results['ERRMSG'] != '') {
          $alarmMsg = "Latest history log failed";
          $alarmTip = sprintf("At %s : %s", $results['STARTTIME'],$results['ERRMSG']);
          $this->alarms[] = new FTSAlarm($alarmMsg, E_ERROR, $alarmTip);
        }
      }
    }
  }
}
