<?php
/*
ftsutils.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
VERSION: '$Id: ftsutils.php,v 1.30 2011/02/17 11:08:40 schwarz Exp $';
AUTHOR: Lionel Schwarz (lionel.schwarz@in2p3.fr)
*/

class CacheItem {
	var $name;
	var $fileName;
	function __construct($name) {
		global $ftsm;
		$this->name = $name;
		$this->fileName = $ftsm->conf->cache_dir . '/' . md5($name);
	}
	
	function getFileName() {
		return $this->fileName;
	}
	
	function isCached($lifetime=null) {
		global $ftsm;
		if (is_null($lifetime)) $lifetime=$ftsm->conf->refresh;
		if (!file_exists($this->fileName) || filemtime($this->fileName)<time()-$lifetime)
			return FALSE;
		return TRUE;
   }
}

class CacheImg extends CacheItem {
	function __construct($name) {
		parent::__construct($name);
		$this->fileName .= '.png';
	}
}
		
function ftsSyslog($msg, $level=LOG_NOTICE) {
  global $ftsdebug;
  if ($ftsdebug or $level < LOG_NOTICE) syslog($level,$msg);
}

function ftsCacheReadQuery($query, $refresh) {
  return ftsCacheRead(md5($query), $refresh);
}

function ftsCacheWriteQuery($query, $obj) {
  return ftsCacheWrite(md5($query), $obj);
}

function ftsCacheRead($cachefile, $refresh) {
  global $ftsm;
  if ($refresh < 0) throw new Exception('CACHE_NOT_REQUESTED');
  $cache = $ftsm->conf->cache_dir . '/' . $cachefile;
  $expire = time()-$refresh;

  if (file_exists($cache) && filemtime($cache)>$expire) {
    $cont = file_get_contents($cache);
    if ($cont === False) {
      throw new Exception('ERROR_READING_CACHE');
    } else {
      //return unserialize(file_get_contents($cache));
      $unser = unserialize($cont);
      if ($unser === False) {
        throw new Exception('ERROR_READING_CACHE');
      }
      return $unser;
    }
  }
  throw new Exception('NOT_IN_CACHE');
}

function ftsCacheWrite($cachefile, $obj) {
  global $ftsm;
  $cache = $ftsm->conf->cache_dir . '/' . $cachefile;
  $serialized = serialize($obj);
  $nbytes = file_put_contents($cache, $serialized);
  if ($nbytes === False) {
    ftsSyslog(sprintf("Could not write cache %s\n", $cache), LOG_WARNING);
  } else {
    ftsSyslog(sprintf("Wrote %d bytes to %s\n", $nbytes, $cache), LOG_DEBUG);
  }
}

function ftsCacheGetImgName($n) {
	global $ftsm;
	return $ftsm->conf->cache_dir . '/' . md5($n) . '.png';
}

function ftsCacheClear($refresh=86400) {
  global $ftsm;
  $cachefile = $ftsm->conf->cache_dir . '/clear';
  // clearstatcache(True, $cachefile) is not avalaible before 5.3.0
  //clearstatcache(True, $cachefile);
  $expire = time()-$refresh;
  $clear_st=stat($cachefile);

  if ($clear_st != False) {
    if ($clear_st['mtime'] >$expire) return;
    foreach (scandir($ftsm->conf->cache_dir) as $f) {
      $f = $ftsm->conf->cache_dir . '/' . $f;
      if (is_file($f) && filectime($f)<$expire && $f != $cachefile && $f != $ftsm->conf->cache_dir . '/.htaccess') {
        $ret = unlink($f);
        if ($ret) ftsSyslog("Removed $f");
        else      ftsSyslog("Error removing Removed $f");
      }
    }
  }
  if (!touch($cachefile)) {
    //Error touch, try to write
    $fp = fopen($cachefile, 'w');
    fwrite($fp, 'clear');
    fclose($fp);
  }
}

function getValidSurl($surl) {
  if (trim($surl) == '') return trim($surl);
  // si srm://xxx.xxx.xxx.xxx:xxxx/srm/managervX?SFN=/...
  //if (preg_match("/srm:\/\/", $surl)==1)  return $surl;
  $surlElements = parse_url($surl);
  //print_r($surlElements);
  if ($surlElements) {
    if ($surlElements['scheme'] != 'srm')
      throw new InvalidInputException("Not a valid SURL: $surl");
    if ($surlElements['path'] == '/srm/managerv1' || $surlElements['path'] == '/srm/managerv2') {
      if (substr( $surlElements['query'] ,0,4) != 'SFN=')
        throw new InvalidInputException("Not a valid SURL: $surl");
      return $surl;
    } else { // Dans le cas srm://xxx:xxx/pnfs/...
      $newsurl = $surlElements['scheme'] . '://';
      $newsurl .= $surlElements['host'];
      if ($surlElements['port'] != '')
        $newsurl .= ':' . $surlElements['port'];
      else
        $newsurl .= ':8443';
      $newsurl .= '/srm/managerv2?SFN=';
      $newsurl .= $surlElements['path'];
      //return $newsurl;
      return trim($surl);
    }
  }
  throw new InvalidInputException("Not a valid SURL: $surl");
}

function isValidJobId($jobid) {
  if (trim($jobid) == '') return true;

  // Check job id like: 1b159557-d627-11dc-bcb5-fd68ca48d132
  return ereg ("([[:xdigit:]]{8})-([[:xdigit:]]{4})-([[:xdigit:]]{4})-([[:xdigit:]]{4})-([[:xdigit:]]{12})", $jobid);
}

function getHumanSize($size_in_bytes) {
    $debit = $size_in_bytes +0;
    $units=Array("B", "KiB", "MiB", "GiB");
    $indice=0;
    while ($debit >= 1024) {
      $debit = $debit/1024.0;
      $indice++;
    }
    return sprintf("%.2f %s", $debit, $units[$indice]);
}

/*
function isValidState($st) {
  global $states;
  return in_array($st, $states);
}
*/
function isValidChannel($channel) {
  if ($channel === FTS_NONE or $channel === FTS_ALL) return True;
  return ereg ("([[:alnum:]]+)-([[:alnum:]]+)", $channel);
}

function isValidFileid($fileid) {
  return (preg_match('/^[0-9]+$/', $fileid) > 0);
}

function isValidTransferState($state) {
  return ($state == FTS_STATE_COMPLETED or $state == FTS_STATE_FAILED);
}

function isValidTransferId($trid) {
  return (preg_match('/^\w+-\w+__\d{4}-\d{2}-\d{2}-\d{4}_\w{6}$/', $trid) > 0);
}

function getTimeInSql($timeColumn, $asColumn=null) {
  global $ftsm;
  // Build AS with column name: remove table name if needed
  if (is_null($asColumn)) {
    $asColumn = array_pop(explode('.',$timeColumn));
  }
  if ($ftsm->conf->time_utc) {
    // SYS_EXTRACT_UTC(timestamp): gives UTC timestamp
    // FROM_TZ: makes timestamp with time zone (+00:00 for UTC)
    // to_char: print with user's format
    return sprintf("to_char(FROM_TZ(SYS_EXTRACT_UTC(%s), '+00:00'),'%s') as %s", 
	$timeColumn, 
	$ftsm->conf->time_format,
	$asColumn);
  } else {
    return sprintf("to_char(%s AT TIME ZONE sessiontimezone, '%s') as %s",
	$timeColumn, 
	$ftsm->conf->time_format,
	$asColumn);
  }
  return $timeColumn;
}


class FTSStat {
  var $name;
  var $nb_jobs; // not necessary jobs !!! TOFIX

  function __construct($name) {
        $this->name = $name;
  }

  function setStat($state, $nb = 0) {
        $this->nb_jobs[$state] = $nb;
  }

  public function toPie($width=350, $height=200) {
    global $ftsm;
    try { // if jpgraph is installed
      if( !function_exists("imagetypes") || !function_exists('imagecreatefromstring') ) throw new Exception("GD is not installed");
      if (!file_exists($ftsm->conf->jpgraph_path . "/src/jpgraph.php")) throw new Exception("JPGraph is not installed");
      require_once ($ftsm->conf->jpgraph_path . "/src/jpgraph.php");
      require_once ($ftsm->conf->jpgraph_path . "/src/jpgraph_pie.php");
      //$graphFileName = ftsCacheGetImgName($this->name . '-pie');
		$plotCacheObj = new CacheImg($this->name . '-pie');
      //if (!file_exists($graphFileName) || filemtime($graphFileName)<time()-1800) {
      if (! $plotCacheObj->isCached($ftsm->conf->refresh)) {
  
        //$data = array(4, 8, 15, 16, 23, 42);
  
        $p = new PiePlot(array_values($this->nb_jobs));
        $colors = Array('#9093e7',
			'white',
			'lightgreen',
			'#ffaa01',
			'red',
			'yellow');
        $p->SetSliceColors($colors); 
        $p->SetLabelType(PIE_VALUE_ABS);
        $p->value->SetFormat('%d');
        $p->SetGuideLines();
        //$p->ExplodeSlice(1); // fait resortir la 2e part
        $p->SetLegends(array_keys($this->nb_jobs));
        $p->SetCenter(0.3,0.5);
  
        $g = new PieGraph($width, $height);
        //$g->title->Set("Les nombres de \"Lost\"");
        $g->Add($p);
        $g->Stroke($plotCacheObj->getFileName());
      }
      return sprintf("<img src='%s'/>", $plotCacheObj->getFileName());
    } catch (Exception $e) {
      throw $e;
    }
  }

}

class ChannelStat extends FTSStat {

  function __construct($name) {
	global $ftsm;
        parent::__construct($name);
        foreach ($ftsm->getStates() as $state) {
          $this->setStat($state);
	}
  }

  public function hasErrors() {
    return ($this->nb_jobs[FTS_STATE_FINISHED_DIRTY] + $this->nb_jobs[FTS_STATE_FAILED] > 0);
  }

  function toString() {
        global $ftsm;
        $nbtot=0;
        foreach ($this->nb_jobs as $nb) {
          $nbtot += $nb;
        }
        $sizetot=500;
        $res = "";
        //print "<div>";
        foreach ($ftsm->getStates() as $state) {
          if ($this->nb_jobs[$state] != 0) {
            //$res .= sprintf("<img title=\"%d %s\" src=\"%s.gif\" width=\"%d\" height=\"10\">", $this->nb_jobs[$state], $state, $state, (int)(($this->nb_jobs[$state]*$sizetot)/$nbtot));
            $perc=($this->nb_jobs[$state]*100)/$nbtot;
            if ($perc >= 10) {
              $res .= sprintf("<div class=\"%s\" style=\"width: %d%%\" >%d</div>", 
                $state,
                $perc,
                $this->nb_jobs[$state]);
            } else {
              $res .= sprintf("<div class=\"%s\" style=\"width: %d%%\" ></div>", 
                $state,
                $perc);
            }
          }
        }
        return $res;
  }


}

